
C======================================================================
      SUBROUTINE getPeakLoads(ivec,iret)
      SAVE
C     ivec= function instruction, usually=1            Last change Oct2013
C     iret= return value 
C     s/r to look at the total stress and remove any non-reversal points
C     from the load history.   Each load set has an integer flag to
C     designate loads to be removed from the memory.  In this program we are
C     focused on the total stress  ststot() to decide removal or keep.

c     Load history storage.  If changing dimensions, also change same 
C                             in mainline
C     Stress membrane, bending, and total storage:
      real*4 stsm(5000),stsb(5000),ststot(5000)
      integer*4 iloadflag(5000) ! 1=save 2=delete  n=repeat
      real*4 ststotmax,ststotmin,ststotwindow
      integer*4 nloads
      logical debugLoads
      common /LOADS/ stsm,stsb,ststot,iloadflag, ststotmax,ststotmin,
     &               ststotwindow,nloads,debugLoads

      real*4 stsold ! the last reversal point
      real*4 stshere ! the present position
      real*4 stsnext ! the next point being examined

      ngood=0                 ! records the no. of good points
      n=0                     ! points to the data being examined
      nbad=0                  ! counts the no. of points being eliminated

C     Get the first point.
      n=n+1
      stsold=ststot(1)
      nstsold=1
      ngood=ngood+1

C     check if the first and last point in the history are same
C     Do this after filtering
      if(ststot(1).eq.ststot(nloads) )then
        write(0,90)
        write(6,90)
   90   format("#history #First and last pts in history are same."/
     &         "#history #Eliminating the 1st history point now")
        nbad=nbad+1
        iloadflag(1)=0
C       Save a copy for rainflow file created near end of program
        ststot1=ststot(1) ! Not used. Adds an extra ramp per block!
      endif

C     Get the second point   stspres  to create the initial line.
  100 continue
      n=n+1
      stsnext=ststot(n)    !get the 2nd point
      nstspres=n

      if(stsnext .eq. stsold)then ! remove the new point
        nbad=nbad+1
        iloadflag(nstspres)=0
        write(6,*)"#DeleteLd samePoint: ",nstspres,stspres
        go to 100
      endif


C     1st (stsold) and 2nd pts (stspres) have been read in----------------
C     2nd pt is not equal to 1st.  Establish direction
      iupdown=+1
      if(stsnext .lt. stsold)iupdown=-1
C     stspres is acceptable
      if(debugLoads)write(6,*)"#stsold,nstsold,stspres,nstspres",
     &               stsold,nstsold,stspres,nstspres,iupdown

C     ok, first point and direction is set. Keep going
 2000 continue
      n=n+1
      if(n .gt. nloads)go to 9000
      stsnext=ststot(n)
      nstsnext=n
      if(debugLoads)write(6,*)"#stsnext,nstsnext: ",stsnext,nstsnext,
     &             iupdown
C     If its equal to the old one we can eliminate the new point
      if(stsnext .eq. stspres)then
        nbad=nbad+1
        iloadflag(nstsnext)=0
        write(6,*)"#DeleteLd samePoint: ",nstsnext stsnext
        go to 2000
      endif

C     No, its not equal.  Could be a reversal  or a continuation of ramp.

      if(stsnext .lt. stspres)go to 3000  ! Potentially it is a reversal point, downwards
C     If not, then things are going in the same direction  if iupdown =1

C     New is going upwards--------------------------------------------------
C     If the same direction we can discard stspres  
      if(iupdown .eq. +1)then
C        Same direction but further, discard pres pt
      if(debugLoads)write(6,*)"#discarding pres pt no.",
     &                 nstspres," iud= ",iupdown
         nbad=nbad+1
         iloadflag(nstspres)=0
        write(6,*)"#Delete non-revPoint: ",nstspres,stspres
         stspres=stsnext
         nstspres=nstsnext
         go to 2000 ! get next point
       endif

C      iupdown must have been = -1,  thus previous direction was in compression.
C      If we get to this point then it is a potential reversal.
C      See if its size is bigger than the allowable window
       if((stsnext-stspres) .lt. ststotwindow)then  ! too small, skip new pt
         nbad=nbad+1
         iloadflag(nstsnext)=0
        write(6,*)"#Delete tooSmallPoint: ",nstsnext,stsnext
         go to 2000  !go get a new one
       endif

C      Ok, it is bigger than the window. Must be a reversal
       if(debugLoads)write(6,*)"#Rev. C_T: old, pres, next ",
     &        stsold,nstsold,stspres,nstspres,stsnext,nstsnext
       ngood=ngood+1
       iupdown=+1   !change direction
C      Set up the reversal point as old
       stsold=stspres
       nstsold=nstspres
       stspres=stsnext
       nstspres=nstsnext
       go to 2000

C     New is going downwards -------------------------------------------------------
 3000 continue
C     the new point is going downwards.  Check the old direction
      if(iupdown .eq. -1)then  ! same direction
C       Discard the old point, and replace it with the new pt.
         nbad=nbad+1
         iloadflag(nstspres)=0
        write(6,*)"#Delete non-revPoint: ",nstspres,stspres
         stspres=stsnext
         nstspres=nstsnext
         go to 2000
      endif

C     iupdown was +1, previous direction was into tension
C     Potential change in direction.  Is it bigger than the window
      if((stspres-stsnext) .lt. ststotwindow)then
C       too small, eliminate the new point
         nbad=nbad+1
         iloadflag(nstsnext)=0
        write(6,*)"#Delete tooSmallPoint: ",nstsnext,stsnext
         go to 2000
      endif

C     Ok, reversal is bigger than the window. Its a reversal
      ngood=ngood+1
      if(debugLoads)write(6,*)"#Rev.T_C: old, pres, next ",
     &   stsold,nstsold,stspres,nstspres,stsnext,nstsnext
      iupdown=-1
      stsold=stspres
      nstsold=nstspres
      stspres=stsnext
      nstspres=nstsnext
      go to 2000

 9000 continue
      if(nbad.eq.0)then    ! no points eliminated
        write(0,9002)
        write(6,9002)
 9002   format("#history #No points needed to be eliminated."/)
        ngood=nloads
        go to 9400  ! print out the filtered history and rainflow 
      endif

      write(0,9005)ngood,nbad
      write(6,9005)ngood,nbad
 9005 format("#history #PPICK found ",i7,"good pts, and ",i7,
     &       " to be eliminated. See list above.")

C     ------------------selction is done.  Must now eliminate and update list

 9400 continue
C     Now go throught the stress storage and take out those to be eliminated
C     Start at the first point to be eliminated
      iput=0        ! index points to next available (empty) storage point
 9500 continue
      iput=iput+1
      if(iloadflag(iput) .ne. 0)goto 9500
C      write(6,*)"#empty at= ",iput  !debug
C     yes flag is 0,  this spot is open for a good pt.

C     Now find the next pt that needs to be saved
      jnextsavept=iput
 9520 continue
      jnextsavept=jnextsavept+1
      if(jnextsavept .gt. nloads)goto 9600  ! end of data found
      if(iloadflag(jnextsavept) .eq. 0)goto 9520
C      write(6,*)"#nextFull at= ",jnextsavept  !debug

C     Yes, we have the next point with iloadflag=1
C     Move this next point to the available postition at iput
      stsm(iput)=stsm(jnextsavept)
      stsb(iput)=stsb(jnextsavept)
      ststot(iput)=ststot(jnextsavept)
      iloadflag(iput)=iloadflag(jnextsavept)
      iloadflag(jnextsavept)=0   !set the moved pt flag to 0
C     For Pdprop  crack propagation programs:
C      write(6,9530)stsm(iput),stsb(iput),ststot(iput),
C     &     iloadflag(iput),iput
C 9530 format("#history #FilteredOut",3(f7.1,1x),i6,1x,i6)
      goto 9500

 9600 continue
C     End of data encountered.
C     Right now iput is still sitting on a flag=0 pt.
      iput=iput-1  ! point back to last non-zero flag point.

C     End-Begin Alignment Check-----------------------------------
C     Check if the first and last point in the history are same
C     Do this after filtering
      nstart=1  !  This is starting point of history, Change?
      if(ststot(1).eq.ststot(nloads) )then
        nstart=2
        write(0,9604)ststot(1),ststot(nstart)
        write(6,9604)ststot(1),ststot(nstart)
 9604   format("#history #1st and Last pts in Filtered hist are same."/
     &         "#history # ststot(1)= ",f8.2/
     &         "#history #Eliminating the 1st history point now"/
     &         "#history #New start pt: ststot()= ",f8.2)
C       Save a copy for rainflow file created near end of program
        ststot1=ststot(2) ! Not used. Adds an extra ramp per block!
      endif

C     There is still the problem of aligning the begin/end of the
C     filtered history.  The PDlisting process does not like two 
C     points in the same direction, one after the other.
C     The method is to look at the end slope, the joint slope, and
C     the begin slope.  If any of these three have "two in a row"
C     with the same sign, we have an alignment problem.
      iwrap=1   ! =1 means Ok,  =0 means problem
      iendDir= +1
      if( (ststot(iput) - ststot(iput-1)) .lt. 0.)iendDir=-1
      ijoinDir= +1
      if( (ststot(nstart) - ststot(iput)) .lt. 0.)ijoinDir=-1
      ibeginDir=  +1
      if( (ststot(nstart+1) - ststot(nstart)) .lt. 0.)ibeginDir=-1

      write(0,*)"#ststot(nstart): ",ststot(nstart), nstart
      write(0,*)"#ststot(nstart+1): ",ststot(nstart+1), nstart+1
      write(0,*)"#ststot(iput-1): ",ststot(iput-1), iput-1
      write(0,*)"#ststot(iput): ",ststot(iput), iput
      write(0,*)"#iendDir: ",iendDir
      write(0,*)"#ijoinDir: ",ijoinDir
      write(0,*)"#ibeginDir: ",ibeginDir

      if(iendDir .eq. ijoinDir)iwrap=0
      if(ijoinDir .eq. ibeginDir)iwrap=0
      if(iwrap .eq. 0)then
        write(0,9608)ststot(iput-1),ststot(iput),ststot(nstart),
     &               ststot(nstart+1)
        write(6,9608)ststot(iput-1),ststot(iput),ststot(nstart),
     &               ststot(nstart+1)
 9608   format("#history #Warning: After filtering: End of history and",
     &         "Begining have an alignment problem:"/
     &         "#history # Last-1 Pt:  ",f8.2/
     &         "#history # Last   Pt:  ",f8.2/
     &         "#history # 1st    Pt:  ",f8.2/
     &         "#history # 2nd    Pt:  ",f8.2)
        
C       Check if all three same direction
        if((iendDir.eq.ijoinDir).and.(ijointDir.eq.ibeginDir) )then
          write(0,9612)
          write(6,9612)
 9612     format("#history # All in same direction.  Eliminate 1st and",
     &           " last points.")
          nstart=nstart +1 ! get rid of 1st pt.
          iput=iput-1      ! get rid of last pt.
          goto 9620
        endif

        if(iendDir .eq. ijoinDir)then
          write(0,9616)
          write(6,9616)
 9616     format("#history # Last and Join in same direction. ",
     &           "Eliminating the Last point.")
          iput=iput-1
          goto 9620
        endif

        if(ijoinDir .eq. ibeginDir)then
          write(0,9618)
          write(6,9618)
 9618     format("#history # Join and Begin are same direction.",
     &           "Eliminating the First point.")
          nstart=nstart+1
          goto 9620
        endif
      endif
C     Thinking about this some more:  It would probably achieve the
C     same effect by adding the 1st point  onto the end of the pre-filter
C     history and then filtering.   (too late now :)
C     Perhaps it is better to check "properly" as above anyway.

 9620 continue
      write(0,9630)ngood,iput
      write(6,9630)ngood,iput
 9630 format("#history #BugCheck: GoodPtsFound=",i8,
     &      " GoodPtsSaved=",i8," should be nearly equal?.")

C     When nstart is not equal to 1  anymore we need to shift the
C     history such that the start point, presently = nstart,  is pt 1

C     Check results with a full print
      icount=0
      do 9800 i=nstart,iput
        icount=icount+1
        write(6,9790)stsm(i),stsb(i),ststot(i),iloadflag(i),icount
 9790   format("#history #Filteredck ",3(f7.1,1x),i8,1x,i6)
        if(nstart .ne. 1)then
C         Shift the history to compensate for nstart > 1
          stsm(icount)     =stsm(i)
          stsb(icount)     =stsb(i)
          ststot(icount)   =ststot(i)
          iloadflag(icount)=iloadflag(i)
        endif
 9800 continue
C     If history has been shifted for non-zero nstart we need
      if(nstart .ne. 1)then
         nstart=1
      endif

C     Also write out a file for rainflow count + StrainStrainLife
      open(unit=10,file="loads4rain.out")
      write(10,9838)ststotmax,ststotmin
 9838 format("#MAX= ",e14.7/"#MIN= ",e14.7/"#BEGIN")
C      write(10,9839)ststot1   ! not a good idea. Distorts history.
C 9839 format(" 0 ",f7.1)
      do 9850 i=1,icount
        write(10,9840)i,ststot(i)
 9840   format(i9,1x,f7.1,1x)
 9850 continue
      close(unit=10)

      write(0,9854)icount,icount
      write(6,9854)icount,icount
 9854 format("#NLOADSETS= ",i8," (loads in each history repetition ",
     $ "after filtering"/"#Wrote TotStress=(Sm+Sb) out for rainflow. ",
     $ "nloads= ",i8,"  into file: loads4rain.out")

      nloads=icount
      return
      end

