C getRevDamage4.f  vers. 2.0  FAC June 2019
C Extract a set of reversals from fadInput.rand  file for plotting damage
C  Compile:  gfortran  -g -w -fbounds-check getRevDamage4.f  -o getRevDamage4
C  Usage:   getRevDamage4 n1 n2 Sy  >outputFile
C           where n1 n2 are begin and end reversal numbers of set.

C   The inputfile is a random access (direct) recl=36 file.

C  The program is made availble to help students develop advances in crack
C  propagation software.

C  Copyright (C) 2019  Al Conle
C This file is free software; you can redistribute it and/or modify
C it under the terms of the GNU General Public License as published by
C the Free Software Foundation; either version 2 of the license, or (at
C your option) any later version.
C  This  file is distributed in the hope that it will be useful, but
C WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTA-
C BILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
C License for more details.
C  You should have received a copy of the GNU General PUblic License along
C with this program; if not, write to the Free Software Foundation, Inc.,
C 59 Temple Place -Suite 330, Boston, MA 02111-1307, USA. Try also their
C web site: http://www.gnu.org/copyleft/gpl.html

C vers. 2.0  fadInput.rand has changed to save  damagePerRev Jun11 2019
C vers. 1.3  Fix small -ve damage due to roundoff(?)  The value of 
C            totdam90 is derived from a double in crack prop program.
C            Seems to be a floating point precison problem.
C            Add calling args output at top of stdo file. Jun8 2019
C vers. 1.2  Add in code for plastic zone size  wpz1, wpz2 Jun1 2019
C fork from plateThruFAD.f


C   Expected input binary format:
C  The numbers were printed by plateThruFlaw.f  statement:
C        nrecord=nrecord+1
C        write(60,rec=nrecord)nrev,totdam90,nblk,nact,
C     &             lobj90,xfw,
C     &             stsMembrane,stsBending

C      Keep same variable names as in plateThruFAD.f for record reads

      character*300  inp300,jnp300      ! used to read in lines as chars.
      character*1    inpone(300),jnpone(300)
      character*10   inpten(30)
      equivalence (inpone(1), inpten(1), inp300)
      equivalence (jnpone(1),jnp300)

      character*30 firstfield, ctail
      character*80 probType
      integer*4 iargc, nargc
      character*80 argv

      character*30 ctypefad   ! reads in which type of FAD data is.

      real*4 totdam90,lobj90,damagePerRev,
     &             xMm90,xMb90,
     &             xMkm90,xMkb90,xfw,
     &             stsMembrane,stsBending
      integer*4  nrev,nblk,nact,nrecord

      pi=3.1415927

      write(0,180)
      write(6,180)
  180 format("# getRevDamage4.f  vers. 2.0 starts...")

C     Extract the begin Rev and end Rev numbers from calling args.

      if(iargc().NE. 3)then
           write(0,*) " Error, not enough args, should be: ",
     &     " getRevDamage  irev   jrev Syield   >out"
           stop
      endif

      call getarg(1,argv)
      read(argv,*) irev   ! start rev no.
      call getarg(2,argv)
      read(argv,*) jrev   ! end rev no.
      call getarg(3,argv)
      read(argv,*) Syield   ! yield stress for pls zone size
      twoSy= 2.0* Syield
      write(6,185)irev,jrev,Syield
  185 format("#StartRev= ",i12/
     &       "#EndRev= ",i12/
     &       "#Syield= ",f6.1  )

C------------------  opn the fadInput.rand file
      open(unit=60, file="fadInput.rand", access="direct",
     &     recl=36, status="old")
      irec=1
      read(60,rec=irec)nrev,totdam90,nblk,nact,
     &             lobj90,xfw,
     &             stsMembrane,stsBending
C     In this first record  nrev   should actually be the number of
C     -ve records written in the file, including this first one.
C     Also, as a check,  totdam90 should be 0.0  and nrev should have
C     been saved as a -ve number (no. of records)
      if(totdam90 .ne. 0.0  .or.
     &   nrev .ge. 0)then
C         something is wrong with this random access file
          write(0,1010)nrev,totdam90
          write(6,1010)nrev,totdam90
 1010     format("# Error: 1st rec. of file fadInput.rand  is wrong:"/
     &           "# nrev = ",i10," (should be -ve)"/
     &           "#    a = ",e14.7," should be = 0.0"/
     &           "# Will try to continue ...")
         goto 1019
      endif
      nrev=-nrev
      maxrecords=nrev

C     If we get to here the 1st rec is ok.  Go read the last rec. and
C     determine what the total reversals of the test was:
      read(60,rec=maxrecords)nrevmax,totdam90,damagePerRev,nblk,nact,
     &             lobj90,xfw,
     &             stsMembrane,stsBending
      write(0,1012)maxrecords,nrevmax,nrevmax,nblk,nact
      write(6,1012)maxrecords,nrevmax,nrevmax,nblk,nact
 1012 format("#MAXRECORDS= ",i10," In last rec. Reversal= ",i10/
     &       "#MAXREVERSALS= ",i10/
     &       "#MAXBLOCKS= ",i10/
     &       "#NACT= ",i10)

C----------------------  get the desired records & write out ---------

 1019 continue

      write(6,1020)
 1020 format("#  HalfCycle    deltaA           A           nblk       ",
     &     " nact    lobj    xfw      Sm       Sb     ",
     &     "wpz1_S      wpz2_deltaS")
C     read the record before rev  irev (which is actually irev since
C     record=2 is for  half-cycle 1 )
C     To get damage before rev irev  in order to compute delta damage.
      if(irev .ne. 1)then
        read(60,rec=irev)nrev, totdam90, damagePerRev, nblk, nact,
     &             lobj90, xfw, stsMembrane, stsBending
        oldStsM = stsMembrane
        oldStsB = stsBending
        oldS = oldStsM + oldStsB
      else   ! User is asking for 1st block, it started at zero.
        oldStsM= 0.
        oldStsB= 0.
        oldS=0.
      endif
       

      do 2000 irec= irev+1,jrev+1
        read(60,rec=irec)nrev, totdam90, damagePerRev, nblk, nact,
     &             lobj90, xfw, stsMembrane, stsBending

C       Also compute Dugdale plastic zone size for Smax and deltaS
C       Hasegawa: "I am interested in plastic zone size wpz: Plastic zone size wpz
C       at a through wall crack in infinite body is calculated by Dugdale model, 
C       as follows;  wpz  = a*[sec((pi*Smax)/(2*Sy) ) - 1.] 
C       where a is the crack length,Smax is the applied stress, Sy is the yield 
C       stress. How different the sizes between compressive stress (0 to -60MPa) 
C       and large tension-compression stress (50 to -150MPa) "
C       Note: secant(x) =  1./(cos(x)
        stress= (stsMembrane+stsBending)
        absStress= abs(stress)  ! use absolute value
        xsecant= 1.0/cos(  (pi*absStress)/(twoSy)  )
        wpz1= totdam90 *(xsecant - 1.)

C       compute the type 2 delta wpz
        deltaStress= abs(stress-oldS)
        xsecant2 =1.0/cos(  (pi*deltaStress)/(twoSy)  )
        wpz2= totdam90 *(xsecant2 - 1.)

        write(6,1050)nrev,damagePerRev,totdam90,nblk,nact,
     &             lobj90,xfw,stsMembrane,stsBending,wpz1,wpz2
 1050   format(     i12,1x,e16.9,1x,e16.9,1x,i10,1x,i10,1x,
     &              f7.1,1x,e10.4,1x,f6.1,1x,f6.1,1x,e14.7,1x,e14.7)
        oldDamage=totdam90
        oldS= stress
 2000 continue
      close(unit=60)
      stop
      end
j
